from time import sleep

import RPi.GPIO as GPIO


# inverted high (see cathode/anode led)
COLOR_ON = GPIO.LOW
COLOR_OFF = GPIO.HIGH


class LedDebugger:

    def __init__(self, red_pin=2, green_pin=3, blue_pin=4):
        self.red = red_pin
        self.green = green_pin
        self.blue = blue_pin
        
        GPIO.setmode(GPIO.BCM)
        GPIO.setwarnings(False)

        GPIO.setup(red_pin, GPIO.OUT)
        GPIO.setup(green_pin, GPIO.OUT)
        GPIO.setup(blue_pin, GPIO.OUT)

    def __del__(self):
        GPIO.cleanup()

    def failed(self):
        self._set_red()

    def success(self):
        self._set_green()

    def neutral(self):
        self._set_blue()

    def _set_red(self):
        GPIO.output(self.red, COLOR_ON)
        GPIO.output(self.green, COLOR_OFF)
        GPIO.output(self.blue, COLOR_OFF)

    def _set_green(self):
        GPIO.output(self.red, COLOR_OFF)
        GPIO.output(self.green, COLOR_ON)
        GPIO.output(self.blue, COLOR_OFF)

    def _set_blue(self):
        GPIO.output(self.red, COLOR_OFF)
        GPIO.output(self.green, COLOR_OFF)
        GPIO.output(self.blue, COLOR_ON)

    def _off(self):
        GPIO.output(self.red, COLOR_OFF)
        GPIO.output(self.green, COLOR_OFF)
        GPIO.output(self.blue, COLOR_OFF)



import websocket
import requests
import json
import inotify.adapters
import threading

from time import sleep
from getmac import get_mac_address

from .led_Debugger import LedDebugger

DEBUG = LedDebugger()

class WSConnection:

    SWITCH_SUCCESS = json.dumps({'message': 200})

    def __init__(self, url, interface="eth0"):
        self.url = url
        self.interface = interface
        self.actuator_on = False
        self.token = None
        self.socket = websocket.WebSocketApp(self.url,
                                            on_open = self._on_open,
                                            on_message = self._on_message,
                                            on_error = self._on_error,
                                            on_close = self._on_close)

        self.socket_thread = threading.Thread(target=self._run_forever)
        self.socket_thread.daemon = True

    def start_connection(self):
        if not self.socket_thread.is_alive():
            self.socket_thread.start()
            
    def _run_forever(self):
        while True:
            sleep(2)
            print('Trying to connect')
            self.socket.run_forever()

    def _switch_actuator(self):
        # send message to mesh network
        self.actuator_on = not self.actuator_on
    
    def _on_open(self):
        print('Connected')
        DEBUG.success()

    def _on_message(self, message):
        print('Switching actuator')
        DEBUG.neutral()
        sleep(1)
        self._switch_actuator()
        self.socket.send(self.SWITCH_SUCCESS)
        DEBUG.success()

    def _on_error(self, error):
        print('Error "' + error + '"')
        DEBUG.failed()
        sleep(5)
        print('Retrying')
        self.start_connection()

    def _on_close(self):
        print("Connection lost")
        DEBUG.failed()
        sleep(5)
        print('Trying to stabilish connection')
        self.start_connection()


"""
Send data to the API.
"""
def send_http_data(url: str, token: str, data: dict):
    payload = {'Authorization': token}
    msg = json.dumps(data)
    response = requests.post(url, data=payload, json=msg)
    
    return response.status

def monitor(directory, url, mac_addr):
    watch_thread = threading.Thread(target=watch_for_collects, args=(directory,))    
    token_thread = threading.Thread(target=get_token, args=(url, mac_addr))
    
    watch_thread.start()
    token_thread.start()

"""
Watch for new collects and send to the API.
"""
def watch_for_collects(directory: str):
    i = inotify.adapters.Inotify()

    i.add_watch(directory)

    for event in i.event_gen():
        if event:
            event_type = event[1]
            if event_type[0] == 'IN_CLOSE_WRITE':
                print('path ' + event[2])
                print('name ' + event[3])

"""
Constantly makes get requests to get token
"""
def get_token(url: str, mac_addr: str):
    formated_mac = mac_addr.replace(':', '-')
    payload = {'mac': formated_mac}
    response = requests.get(url, params=payload)

    while response.status_code != 200:
        print('Trying to acquire token')
        sleep(10)
        response = requests.get(url, params=payload)
    
    DEBUG.neutral()
    sleep(1)
    f = open('/home/ricc/token', 'w')
    f.write(response.text)
    DEBUG.success()


"""
Let the server know the central is online
"""
def signup(url: str, mac_addr: str):
    headers = {"content-type": "application/json"}
    msg = json.dumps({'mac_address': mac_addr})
    response = requests.post(url, data=msg, headers=headers)
    
    if response.status_code == 201:
        DEBUG.neutral()
        sleep(1)
        DEBUG.success()

    return response



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

setup(
    name='old-major',

    version='0.0.11',

    description='RICC project central system',

    url='https://github.com/ricc-project/old-major',

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    install_requires=['websocket_client', 'getmac', 'requests', 'inotify'],

    entry_points={
        'console_scripts': [
            'old-major = src.old_major:main',
        ],
    },
)



import argparse
import os

from getmac import get_mac_address

from .connection import WSConnection, watch_for_collects, get_token, monitor, signup


def main():
    parser = argparse.ArgumentParser()
    
    parser.add_argument('host', type=str, help='a hostname to make http and tcp requests',
                        nargs='?', default='localhost:8000')
    
    parser.add_argument('interface', type=str, help='internet interface used',
                        nargs='?', default='eth0')

    parser.add_argument('directory', type=str, help='directory where sensor data is saved',
                        nargs='?', default='/home/ricc/data')
    
    args = parser.parse_args()

    mac_addr = get_mac_address(args.interface)
    
    if mac_addr:
        if not os.path.exists(args.directory):
            os.makedirs(args.directory)

        ws_url = 'ws://' + args.host + '/ws/?mac=' + mac_addr
        http_url = 'http://' + args.host
        print('ws ', ws_url)
        print('http ', http_url)
        signup(http_url+'/sign_central/', mac_addr)
        connection = WSConnection(ws_url, args.interface)
        connection.start_connection()
        monitor(args.directory, (http_url + '/call/'), mac_addr)
    else:
        print("Invalid internet interface, couldn't get device mac address")
